require 'test_helper'

class ActsAsIntegerInfinitableTest < ActiveSupport::TestCase
  test "truth" do
    assert_kind_of Module, ActsAsIntegerInfinitable
  end

  def test_a_number_after_saving_should_be_the_same
    subscription = LibrarySubscription.create(available_books: 5)
    assert_equal 5, subscription.reload.available_books
  end

  def test_a_number_should_decrement
    subscription = LibrarySubscription.create(available_books: 5)
    subscription.rent_book
    assert_equal 4, subscription.reload.available_books
  end

  def test_assing_infinity_on_creation_should_be_infinity
    subscription = LibrarySubscription.create(available_books: Float::INFINITY)
    assert_equal Float::INFINITY, subscription.available_books
  end

  def test_infinity_after_decrement_should_not_change
    subscription = LibrarySubscription.create(available_books: Float::INFINITY)
    subscription.rent_book
    assert_equal Float::INFINITY, subscription.reload.available_books
  end

  def test_assing_infinity_on_creation_should_be_infinity_after_reloading
    subscription = LibrarySubscription.create(available_books: Float::INFINITY)
    assert_equal Float::INFINITY, subscription.reload.available_books
  end

  def test_assign_infinity_to_existing_instace_should_be_infinity_after_reloading
    subscription = LibrarySubscription.create
    subscription.available_books = Float::INFINITY
    subscription.save
    assert_equal Float::INFINITY, subscription.reload.available_books
  end

  def test_works_on_multiple_fields
    tireless_warrior = TirelessWarrior.create(movements_available: Float::INFINITY,
                                              attacks_available: Float::INFINITY)
    assert_equal Float::INFINITY, tireless_warrior.movements_available
    assert_equal Float::INFINITY, tireless_warrior.attacks_available
  end

  def test_can_define_different_value_as_infinity
    poor = Person.create(funds: 0)
    assert_equal 0, poor.funds

    debtor = Person.create(funds: -1)
    assert_equal -1, debtor.funds

    super_millonaire = Person.create(funds: Float::INFINITY)
    assert_equal Float::INFINITY, super_millonaire.funds
  end
end
